@@ -1,4 +1,3 @@
-
 /*
   Defines projective product routines where A is a MPIAIJ matrix
           C = P^T * A * P
@@ -13,17 +12,17 @@
 #include <petsc/private/hashseti.h>
 #include <petscsf.h>
 
-PetscErrorCode MatView_MPIAIJ_PtAP(Mat A, PetscViewer viewer)
+static PetscErrorCode MatView_MPIAIJ_PtAP(Mat A, PetscViewer viewer)
 {
-  PetscBool         iascii;
+  PetscBool         isascii;
   PetscViewerFormat format;
   Mat_APMPI        *ptap;
 
   PetscFunctionBegin;
   MatCheckProduct(A, 1);
   ptap = (Mat_APMPI *)A->product->data;
-  PetscCall(PetscObjectTypeCompare((PetscObject)viewer, PETSCVIEWERASCII, &iascii));
-  if (iascii) {
+  PetscCall(PetscObjectTypeCompare((PetscObject)viewer, PETSCVIEWERASCII, &isascii));
+  if (isascii) {
     PetscCall(PetscViewerGetFormat(viewer, &format));
     if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
       if (ptap->algType == 0) {
@@ -54,7 +53,7 @@ PetscErrorCode MatDestroy_MPIAIJ_PtAP(void *data)
   PetscCall(MatDestroy(&ptap->Rd));
   PetscCall(MatDestroy(&ptap->Ro));
   if (ptap->AP_loc) { /* used by alg_rap */
-    Mat_SeqAIJ *ap = (Mat_SeqAIJ *)(ptap->AP_loc)->data;
+    Mat_SeqAIJ *ap = (Mat_SeqAIJ *)ptap->AP_loc->data;
     PetscCall(PetscFree(ap->i));
     PetscCall(PetscFree2(ap->j, ap->a));
     PetscCall(MatDestroy(&ptap->AP_loc));
@@ -97,7 +96,7 @@ PetscErrorCode MatDestroy_MPIAIJ_PtAP(void *data)
 PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, Mat C)
 {
   Mat_MPIAIJ        *a = (Mat_MPIAIJ *)A->data, *p = (Mat_MPIAIJ *)P->data;
-  Mat_SeqAIJ        *ad = (Mat_SeqAIJ *)(a->A)->data, *ao = (Mat_SeqAIJ *)(a->B)->data;
+  Mat_SeqAIJ        *ad = (Mat_SeqAIJ *)a->A->data, *ao = (Mat_SeqAIJ *)a->B->data;
   Mat_SeqAIJ        *ap, *p_loc, *p_oth = NULL, *c_seq;
   Mat_APMPI         *ptap;
   Mat                AP_loc, C_loc, C_oth;
@@ -133,8 +132,8 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, Mat C)
 
   /* 2-2) compute numeric A_loc*P - dominating part */
   /* get data from symbolic products */
-  p_loc = (Mat_SeqAIJ *)(ptap->P_loc)->data;
-  if (ptap->P_oth) p_oth = (Mat_SeqAIJ *)(ptap->P_oth)->data;
+  p_loc = (Mat_SeqAIJ *)ptap->P_loc->data;
+  if (ptap->P_oth) p_oth = (Mat_SeqAIJ *)ptap->P_oth->data;
 
   api = ap->i;
   apj = ap->j;
@@ -157,7 +156,7 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, Mat C)
   C_loc = ptap->C_loc;
   C_oth = ptap->C_oth;
 
-  /* add C_loc and Co to to C */
+  /* add C_loc and Co to C */
   PetscCall(MatGetOwnershipRange(C, &rstart, &rend));
 
   /* C_loc -> C */
@@ -222,12 +221,12 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
   Mat                      P_loc, P_oth;
   PetscFreeSpaceList       free_space = NULL, current_space = NULL;
   PetscInt                 am = A->rmap->n, pm = P->rmap->n, pN = P->cmap->N, pn = P->cmap->n;
-  PetscInt                *lnk, i, k, pnz, row, nsend;
-  PetscMPIInt              tagi, tagj, *len_si, *len_s, *len_ri, nrecv;
+  PetscInt                *lnk, i, k, pnz, row;
+  PetscMPIInt              tagi, tagj, *len_si, *len_s, *len_ri, nrecv, nsend, proc;
   PETSC_UNUSED PetscMPIInt icompleted = 0;
   PetscInt               **buf_rj, **buf_ri, **buf_ri_k;
   const PetscInt          *owners;
-  PetscInt                 len, proc, *dnz, *onz, nzi, nspacedouble;
+  PetscInt                 len, *dnz, *onz, nzi, nspacedouble;
   PetscInt                 nrows, *buf_s, *buf_si, *buf_si_i, **nextrow, **nextci;
   MPI_Request             *swaits, *rwaits;
   MPI_Status              *sstatus, rstatus;
@@ -235,7 +234,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
   PetscInt                *owners_co, *coi, *coj; /* i and j array of (p->B)^T*A*P - used in the communication */
   PetscMPIInt             *len_r, *id_r;          /* array of length of comm->size, store send/recv matrix values */
   PetscInt                *api, *apj, *Jptr, apnz, *prmap = p->garray, con, j, Crmax, *aj, *ai, *pi, nout;
-  Mat_SeqAIJ              *p_loc, *p_oth = NULL, *ad = (Mat_SeqAIJ *)(a->A)->data, *ao = NULL, *c_loc, *c_oth;
+  Mat_SeqAIJ              *p_loc, *p_oth = NULL, *ad = (Mat_SeqAIJ *)a->A->data, *ao = NULL, *c_loc, *c_oth;
   PetscScalar             *apv;
   PetscHMapI               ta;
   MatType                  mtype;
@@ -251,7 +250,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
   PetscCallMPI(MPI_Comm_size(comm, &size));
   PetscCallMPI(MPI_Comm_rank(comm, &rank));
 
-  if (size > 1) ao = (Mat_SeqAIJ *)(a->B)->data;
+  if (size > 1) ao = (Mat_SeqAIJ *)a->B->data;
 
   /* create symbolic parallel matrix Cmpi */
   PetscCall(MatGetType(A, &mtype));
@@ -302,7 +301,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
     ai  = ad->i;
     pi  = p_loc->i;
     nzi = ai[i + 1] - ai[i];
-    aj  = ad->j + ai[i];
+    aj  = PetscSafePointerPlusOffset(ad->j, ai[i]);
     for (j = 0; j < nzi; j++) {
       row  = aj[j];
       pnz  = pi[row + 1] - pi[row];
@@ -315,7 +314,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
       ai  = ao->i;
       pi  = p_oth->i;
       nzi = ai[i + 1] - ai[i];
-      aj  = ao->j + ai[i];
+      aj  = PetscSafePointerPlusOffset(ao->j, ai[i]);
       for (j = 0; j < nzi; j++) {
         row  = aj[j];
         pnz  = pi[row + 1] - pi[row];
@@ -427,7 +426,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
   for (proc = 0, k = 0; proc < size; proc++) {
     if (!len_s[proc]) continue;
     i = owners_co[proc];
-    PetscCallMPI(MPI_Isend(coj + coi[i], len_s[proc], MPIU_INT, proc, tagj, comm, swaits + k));
+    PetscCallMPI(MPIU_Isend(coj + coi[i], len_s[proc], MPIU_INT, proc, tagj, comm, swaits + k));
     k++;
   }
 
@@ -482,7 +481,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
       buf_si[nrows + 1]   = prmap[i] - owners[proc]; /* local row index */
       nrows++;
     }
-    PetscCallMPI(MPI_Isend(buf_si, len_si[proc], MPIU_INT, proc, tagi, comm, swaits + k));
+    PetscCallMPI(MPIU_Isend(buf_si, len_si[proc], MPIU_INT, proc, tagi, comm, swaits + k));
     k++;
     buf_si += len_si[proc];
   }
@@ -496,7 +495,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
   PetscCall(PetscFree(buf_s));
 
   /* (5) compute the local portion of Cmpi      */
-  /* set initial free space to be Crmax, sufficient for holding nozeros in each row of Cmpi */
+  /* set initial free space to be Crmax, sufficient for holding nonzeros in each row of Cmpi */
   PetscCall(PetscFreeSpaceGet(Crmax, &free_space));
   current_space = free_space;
 
@@ -574,7 +573,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat A, Mat P, PetscReal fi
 static inline PetscErrorCode MatPtAPSymbolicComputeOneRowOfAP_private(Mat A, Mat P, Mat P_oth, const PetscInt *map, PetscInt dof, PetscInt i, PetscHSetI dht, PetscHSetI oht)
 {
   Mat_MPIAIJ *a = (Mat_MPIAIJ *)A->data, *p = (Mat_MPIAIJ *)P->data;
-  Mat_SeqAIJ *ad = (Mat_SeqAIJ *)(a->A)->data, *ao = (Mat_SeqAIJ *)(a->B)->data, *p_oth = (Mat_SeqAIJ *)P_oth->data, *pd = (Mat_SeqAIJ *)p->A->data, *po = (Mat_SeqAIJ *)p->B->data;
+  Mat_SeqAIJ *ad = (Mat_SeqAIJ *)a->A->data, *ao = (Mat_SeqAIJ *)a->B->data, *p_oth = (Mat_SeqAIJ *)P_oth->data, *pd = (Mat_SeqAIJ *)p->A->data, *po = (Mat_SeqAIJ *)p->B->data;
   PetscInt   *ai, nzi, j, *aj, row, col, *pi, *pj, pnz, nzpi, *p_othcols, k;
   PetscInt    pcstart, pcend, column, offset;
 
@@ -595,17 +594,17 @@ static inline PetscErrorCode MatPtAPSymbolicComputeOneRowOfAP_private(Mat A, Mat
     pj   = pd->j + pd->i[row];
     for (k = 0; k < nzpi; k++) PetscCall(PetscHSetIAdd(dht, pj[k] * dof + offset + pcstart));
   }
-  /* off diag P */
+  /* off-diagonal P */
   for (j = 0; j < nzi; j++) {
     row    = aj[j];
     offset = row % dof;
     row /= dof;
     nzpi = po->i[row + 1] - po->i[row];
-    pj   = po->j + po->i[row];
+    pj   = PetscSafePointerPlusOffset(po->j, po->i[row]);
     for (k = 0; k < nzpi; k++) PetscCall(PetscHSetIAdd(oht, p->garray[pj[k]] * dof + offset));
   }
 
-  /* off diagonal part: Ao[i, :]*P_oth */
+  /* off-diagonal part: Ao[i, :]*P_oth */
   if (ao) {
     ai  = ao->i;
     pi  = p_oth->i;
@@ -633,7 +632,7 @@ static inline PetscErrorCode MatPtAPSymbolicComputeOneRowOfAP_private(Mat A, Mat
 static inline PetscErrorCode MatPtAPNumericComputeOneRowOfAP_private(Mat A, Mat P, Mat P_oth, const PetscInt *map, PetscInt dof, PetscInt i, PetscHMapIV hmap)
 {
   Mat_MPIAIJ *a = (Mat_MPIAIJ *)A->data, *p = (Mat_MPIAIJ *)P->data;
-  Mat_SeqAIJ *ad = (Mat_SeqAIJ *)(a->A)->data, *ao = (Mat_SeqAIJ *)(a->B)->data, *p_oth = (Mat_SeqAIJ *)P_oth->data, *pd = (Mat_SeqAIJ *)p->A->data, *po = (Mat_SeqAIJ *)p->B->data;
+  Mat_SeqAIJ *ad = (Mat_SeqAIJ *)a->A->data, *ao = (Mat_SeqAIJ *)a->B->data, *p_oth = (Mat_SeqAIJ *)P_oth->data, *pd = (Mat_SeqAIJ *)p->A->data, *po = (Mat_SeqAIJ *)p->B->data;
   PetscInt   *ai, nzi, j, *aj, row, col, *pi, pnz, *p_othcols, pcstart, *pj, k, nzpi, offset;
   PetscScalar ra, *aa, *pa;
 
@@ -663,13 +662,13 @@ static inline PetscErrorCode MatPtAPNumericComputeOneRowOfAP_private(Mat A, Mat
     offset = row % dof;
     row /= dof;
     nzpi = po->i[row + 1] - po->i[row];
-    pj   = po->j + po->i[row];
-    pa   = po->a + po->i[row];
+    pj   = PetscSafePointerPlusOffset(po->j, po->i[row]);
+    pa   = PetscSafePointerPlusOffset(po->a, po->i[row]);
     for (k = 0; k < nzpi; k++) PetscCall(PetscHMapIVAddValue(hmap, p->garray[pj[k]] * dof + offset, ra * pa[k]));
     PetscCall(PetscLogFlops(2.0 * nzpi));
   }
 
-  /* off diagonal part: Ao[i, :]*P_oth */
+  /* off-diagonal part: Ao[i, :]*P_oth */
   if (ao) {
     ai  = ao->i;
     pi  = p_oth->i;
@@ -688,7 +687,6 @@ static inline PetscErrorCode MatPtAPNumericComputeOneRowOfAP_private(Mat A, Mat
       PetscCall(PetscLogFlops(2.0 * pnz));
     }
   } /* end if (ao) */
-
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -774,8 +772,8 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce(Mat A, Mat P, PetscInt do
         }
         PetscCall(PetscLogFlops(voff));
       } /* End jj */
-    }   /* End j */
-  }     /* End i */
+    } /* End j */
+  } /* End i */
 
   PetscCall(PetscFree4(apindices, apvalues, apvaluestmp, c_rmtc));
   PetscCall(PetscHMapIVDestroy(&hmap));
@@ -789,8 +787,8 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce(Mat A, Mat P, PetscInt do
   PetscCall(MatGetOwnershipRangeColumn(P, &pcstart, &pcend));
   pcstart = pcstart * dof;
   pcend   = pcend * dof;
-  cd      = (Mat_SeqAIJ *)(c->A)->data;
-  co      = (Mat_SeqAIJ *)(c->B)->data;
+  cd      = (Mat_SeqAIJ *)c->A->data;
+  co      = (Mat_SeqAIJ *)c->B->data;
 
   cmaxr = 0;
   for (i = 0; i < pn; i++) cmaxr = PetscMax(cmaxr, (cd->i[i + 1] - cd->i[i]) + (co->i[i + 1] - co->i[i]));
@@ -827,7 +825,7 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce(Mat A, Mat P, PetscInt do
   /* Add contributions from remote */
   for (i = 0; i < pn; i++) {
     row = i + pcstart;
-    PetscCall(MatSetValues(C, 1, &row, ptap->c_othi[i + 1] - ptap->c_othi[i], c_othj + ptap->c_othi[i], c_otha + ptap->c_othi[i], ADD_VALUES));
+    PetscCall(MatSetValues(C, 1, &row, ptap->c_othi[i + 1] - ptap->c_othi[i], PetscSafePointerPlusOffset(c_othj, ptap->c_othi[i]), PetscSafePointerPlusOffset(c_otha, ptap->c_othi[i]), ADD_VALUES));
   }
   PetscCall(PetscFree2(c_othj, c_otha));
 
@@ -841,7 +839,6 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce(Mat A, Mat P, PetscInt do
 PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ_allatonce(Mat A, Mat P, Mat C)
 {
   PetscFunctionBegin;
-
   PetscCall(MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce(A, P, 1, C));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -884,8 +881,8 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pets
   pcend *= dof;
   cmaxr = 0;
   for (i = 0; i < pon; i++) cmaxr = PetscMax(cmaxr, ptap->c_rmti[i + 1] - ptap->c_rmti[i]);
-  cd = (Mat_SeqAIJ *)(c->A)->data;
-  co = (Mat_SeqAIJ *)(c->B)->data;
+  cd = (Mat_SeqAIJ *)c->A->data;
+  co = (Mat_SeqAIJ *)c->B->data;
   for (i = 0; i < pn; i++) cmaxr = PetscMax(cmaxr, (cd->i[i + 1] - cd->i[i]) + (co->i[i + 1] - co->i[i]));
   PetscCall(PetscCalloc4(cmaxr, &apindices, cmaxr, &apvalues, cmaxr, &apvaluestmp, pon, &c_rmtc));
   PetscCall(PetscHMapIVCreateWithSize(cmaxr, &hmap));
@@ -903,8 +900,8 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pets
     if (!voff) continue;
 
     /* Form remote C(ii, :) */
-    poj = po->j + po->i[ii];
-    poa = po->a + po->i[ii];
+    poj = PetscSafePointerPlusOffset(po->j, po->i[ii]);
+    poa = PetscSafePointerPlusOffset(po->a, po->i[ii]);
     for (j = 0; j < nzi; j++) {
       pocol   = poj[j] * dof + offset;
       c_rmtjj = c_rmtj + ptap->c_rmti[pocol];
@@ -946,7 +943,7 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pets
       PetscCall(PetscLogFlops(voff));
       PetscCall(MatSetValues(C, 1, &row, voff, apindices, apvaluestmp, ADD_VALUES));
     } /* End j */
-  }   /* End i */
+  } /* End i */
 
   PetscCall(ISRestoreIndices(map, &mappingindices));
   PetscCall(PetscFree4(apindices, apvalues, apvaluestmp, c_rmtc));
@@ -962,7 +959,7 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pets
   /* Add contributions from remote */
   for (i = 0; i < pn; i++) {
     row = i + pcstart;
-    PetscCall(MatSetValues(C, 1, &row, ptap->c_othi[i + 1] - ptap->c_othi[i], c_othj + ptap->c_othi[i], c_otha + ptap->c_othi[i], ADD_VALUES));
+    PetscCall(MatSetValues(C, 1, &row, ptap->c_othi[i + 1] - ptap->c_othi[i], PetscSafePointerPlusOffset(c_othj, ptap->c_othi[i]), PetscSafePointerPlusOffset(c_otha, ptap->c_othi[i]), ADD_VALUES));
   }
   PetscCall(PetscFree2(c_othj, c_otha));
 
@@ -976,7 +973,6 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pets
 PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ_allatonce_merged(Mat A, Mat P, Mat C)
 {
   PetscFunctionBegin;
-
   PetscCall(MatPtAPNumeric_MPIAIJ_MPIXAIJ_allatonce_merged(A, P, 1, C));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -1044,7 +1040,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIXAIJ_allatonce(Mat A, Mat P, PetscInt d
     PetscCall(PetscHSetIClear(ht));
     offset = i % dof;
     ii     = i / dof;
-    /* If the off diag is empty, we should not do any calculation */
+    /* If the off-diagonal is empty, do not do any calculation */
     nzi = po->i[ii + 1] - po->i[ii];
     if (!nzi) continue;
 
@@ -1188,13 +1184,13 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIXAIJ_allatonce(Mat A, Mat P, PetscInt d
 
   for (i = 0; i < pn; i++) {
     nzi = ptap->c_othi[i + 1] - ptap->c_othi[i];
-    rdj = c_othj + ptap->c_othi[i];
+    rdj = PetscSafePointerPlusOffset(c_othj, ptap->c_othi[i]);
     for (j = 0; j < nzi; j++) {
       col = rdj[j];
       /* diag part */
       if (col >= pcstart && col < pcend) {
         PetscCall(PetscHSetIAdd(hta[i], col));
-      } else { /* off diag */
+      } else { /* off-diagonal */
         PetscCall(PetscHSetIAdd(hto[i], col));
       }
     }
@@ -1318,7 +1314,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pet
     PetscCall(PetscHSetIClear(oht));
     offset = i % dof;
     ii     = i / dof;
-    /* If the off diag is empty, we should not do any calculation */
+    /* If the off-diagonal is empty, do not do any calculation */
     nzi  = po->i[ii + 1] - po->i[ii];
     dnzi = pd->i[ii + 1] - pd->i[ii];
     if (!nzi && !dnzi) continue;
@@ -1330,7 +1326,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pet
     if (!(htsize + htosize)) continue;
 
     /* Form remote C(ii, :) */
-    poj = po->j + po->i[ii];
+    poj = PetscSafePointerPlusOffset(po->j, po->i[ii]);
     for (j = 0; j < nzi; j++) {
       PetscCall(PetscHSetIUpdate(hta[poj[j] * dof + offset], ht));
       PetscCall(PetscHSetIUpdate(hta[poj[j] * dof + offset], oht));
@@ -1445,13 +1441,13 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIXAIJ_allatonce_merged(Mat A, Mat P, Pet
   pcend *= dof;
   for (i = 0; i < pn; i++) {
     nzi = ptap->c_othi[i + 1] - ptap->c_othi[i];
-    rdj = c_othj + ptap->c_othi[i];
+    rdj = PetscSafePointerPlusOffset(c_othj, ptap->c_othi[i]);
     for (j = 0; j < nzi; j++) {
       col = rdj[j];
-      /* diag part */
+      /* diagonal part */
       if (col >= pcstart && col < pcend) {
         PetscCall(PetscHSetIAdd(htd[i], col));
-      } else { /* off diag */
+      } else { /* off-diagonal */
         PetscCall(PetscHSetIAdd(hto[i], col));
       }
     }
@@ -1509,15 +1505,15 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
   Mat_APMPI               *ptap;
   Mat_MPIAIJ              *a = (Mat_MPIAIJ *)A->data, *p = (Mat_MPIAIJ *)P->data;
   MPI_Comm                 comm;
-  PetscMPIInt              size, rank;
+  PetscMPIInt              size, rank, nsend, proc;
   PetscFreeSpaceList       free_space = NULL, current_space = NULL;
   PetscInt                 am = A->rmap->n, pm = P->rmap->n, pN = P->cmap->N, pn = P->cmap->n;
-  PetscInt                *lnk, i, k, pnz, row, nsend;
+  PetscInt                *lnk, i, k, pnz, row;
   PetscBT                  lnkbt;
   PetscMPIInt              tagi, tagj, *len_si, *len_s, *len_ri, nrecv;
   PETSC_UNUSED PetscMPIInt icompleted = 0;
   PetscInt               **buf_rj, **buf_ri, **buf_ri_k;
-  PetscInt                 len, proc, *dnz, *onz, *owners, nzi, nspacedouble;
+  PetscInt                 len, *dnz, *onz, *owners, nzi, nspacedouble;
   PetscInt                 nrows, *buf_s, *buf_si, *buf_si_i, **nextrow, **nextci;
   MPI_Request             *swaits, *rwaits;
   MPI_Status              *sstatus, rstatus;
@@ -1525,7 +1521,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
   PetscInt                *owners_co, *coi, *coj; /* i and j array of (p->B)^T*A*P - used in the communication */
   PetscMPIInt             *len_r, *id_r;          /* array of length of comm->size, store send/recv matrix values */
   PetscInt                *api, *apj, *Jptr, apnz, *prmap = p->garray, con, j, ap_rmax = 0, Crmax, *aj, *ai, *pi;
-  Mat_SeqAIJ              *p_loc, *p_oth = NULL, *ad = (Mat_SeqAIJ *)(a->A)->data, *ao = NULL, *c_loc, *c_oth;
+  Mat_SeqAIJ              *p_loc, *p_oth = NULL, *ad = (Mat_SeqAIJ *)a->A->data, *ao = NULL, *c_loc, *c_oth;
   PetscScalar             *apv;
   PetscHMapI               ta;
   MatType                  mtype;
@@ -1541,7 +1537,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
   PetscCallMPI(MPI_Comm_size(comm, &size));
   PetscCallMPI(MPI_Comm_rank(comm, &rank));
 
-  if (size > 1) ao = (Mat_SeqAIJ *)(a->B)->data;
+  if (size > 1) ao = (Mat_SeqAIJ *)a->B->data;
 
   /* create symbolic parallel matrix Cmpi */
   PetscCall(MatGetType(A, &mtype));
@@ -1565,8 +1561,8 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
   PetscCall(MatTranspose(p->B, MAT_INITIAL_MATRIX, &ptap->Ro));
 
   /* (1) compute symbolic AP = A_loc*P = Ad*P_loc + Ao*P_oth (api,apj) */
-  p_loc = (Mat_SeqAIJ *)(ptap->P_loc)->data;
-  if (ptap->P_oth) p_oth = (Mat_SeqAIJ *)(ptap->P_oth)->data;
+  p_loc = (Mat_SeqAIJ *)ptap->P_loc->data;
+  if (ptap->P_oth) p_oth = (Mat_SeqAIJ *)ptap->P_oth->data;
 
   /* create and initialize a linked list */
   PetscCall(PetscHMapICreateWithSize(pn, &ta)); /* for compute AP_loc and Cmpi */
@@ -1593,7 +1589,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
     ai  = ad->i;
     pi  = p_loc->i;
     nzi = ai[i + 1] - ai[i];
-    aj  = ad->j + ai[i];
+    aj  = PetscSafePointerPlusOffset(ad->j, ai[i]);
     for (j = 0; j < nzi; j++) {
       row  = aj[j];
       pnz  = pi[row + 1] - pi[row];
@@ -1606,11 +1602,11 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
       ai  = ao->i;
       pi  = p_oth->i;
       nzi = ai[i + 1] - ai[i];
-      aj  = ao->j + ai[i];
+      aj  = PetscSafePointerPlusOffset(ao->j, ai[i]);
       for (j = 0; j < nzi; j++) {
         row  = aj[j];
         pnz  = pi[row + 1] - pi[row];
-        Jptr = p_oth->j + pi[row];
+        Jptr = PetscSafePointerPlusOffset(p_oth->j, pi[row]);
         PetscCall(PetscLLCondensedAddSorted(pnz, Jptr, lnk, lnkbt));
       }
     }
@@ -1627,7 +1623,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
     /* Copy data into free space, then initialize lnk */
     PetscCall(PetscLLCondensedClean(pN, apnz, current_space->array, lnk, lnkbt));
 
-    current_space->array += apnz;
+    current_space->array = PetscSafePointerPlusOffset(current_space->array, apnz);
     current_space->local_used += apnz;
     current_space->local_remaining -= apnz;
   }
@@ -1719,7 +1715,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
   for (proc = 0, k = 0; proc < size; proc++) {
     if (!len_s[proc]) continue;
     i = owners_co[proc];
-    PetscCallMPI(MPI_Isend(coj + coi[i], len_s[proc], MPIU_INT, proc, tagj, comm, swaits + k));
+    PetscCallMPI(MPIU_Isend(coj + coi[i], len_s[proc], MPIU_INT, proc, tagj, comm, swaits + k));
     k++;
   }
 
@@ -1774,7 +1770,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
       buf_si[nrows + 1]   = prmap[i] - owners[proc]; /* local row index */
       nrows++;
     }
-    PetscCallMPI(MPI_Isend(buf_si, len_si[proc], MPIU_INT, proc, tagi, comm, swaits + k));
+    PetscCallMPI(MPIU_Isend(buf_si, len_si[proc], MPIU_INT, proc, tagi, comm, swaits + k));
     k++;
     buf_si += len_si[proc];
   }
@@ -1788,7 +1784,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
   PetscCall(PetscFree(buf_s));
 
   /* (5) compute the local portion of Cmpi      */
-  /* set initial free space to be Crmax, sufficient for holding nozeros in each row of Cmpi */
+  /* set initial free space to be Crmax, sufficient for holding nonzeros in each row of Cmpi */
   PetscCall(PetscFreeSpaceGet(Crmax, &free_space));
   current_space = free_space;
 
@@ -1805,7 +1801,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
   for (i = 0; i < pn; i++) {
     /* add C_loc into Cmpi */
     nzi  = c_loc->i[i + 1] - c_loc->i[i];
-    Jptr = c_loc->j + c_loc->i[i];
+    Jptr = PetscSafePointerPlusOffset(c_loc->j, c_loc->i[i]);
     PetscCall(PetscLLCondensedAddSorted(nzi, Jptr, lnk, lnkbt));
 
     /* add received col data into lnk */
@@ -1861,7 +1857,7 @@ PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ(Mat A, Mat P, PetscReal fill, Mat C
 PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ(Mat A, Mat P, Mat C)
 {
   Mat_MPIAIJ        *a = (Mat_MPIAIJ *)A->data, *p = (Mat_MPIAIJ *)P->data;
-  Mat_SeqAIJ        *ad = (Mat_SeqAIJ *)(a->A)->data, *ao = (Mat_SeqAIJ *)(a->B)->data;
+  Mat_SeqAIJ        *ad = (Mat_SeqAIJ *)a->A->data, *ao = (Mat_SeqAIJ *)a->B->data;
   Mat_SeqAIJ        *ap, *p_loc, *p_oth = NULL, *c_seq;
   Mat_APMPI         *ptap;
   Mat                AP_loc, C_loc, C_oth;
@@ -1897,8 +1893,8 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ(Mat A, Mat P, Mat C)
 
   /* 2-2) compute numeric A_loc*P - dominating part */
   /* get data from symbolic products */
-  p_loc = (Mat_SeqAIJ *)(ptap->P_loc)->data;
-  if (ptap->P_oth) p_oth = (Mat_SeqAIJ *)(ptap->P_oth)->data;
+  p_loc = (Mat_SeqAIJ *)ptap->P_loc->data;
+  if (ptap->P_oth) p_oth = (Mat_SeqAIJ *)ptap->P_oth->data;
   apa = ptap->apa;
   api = ap->i;
   apj = ap->j;
@@ -1921,7 +1917,7 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ(Mat A, Mat P, Mat C)
   C_loc = ptap->C_loc;
   C_oth = ptap->C_oth;
 
-  /* add C_loc and Co to to C */
+  /* add C_loc and Co to C */
   PetscCall(MatGetOwnershipRange(C, &rstart, &rend));
 
   /* C_loc -> C */
@@ -1944,7 +1940,7 @@ PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ(Mat A, Mat P, Mat C)
       ncols = c_seq->i[i + 1] - c_seq->i[i];
       row   = rstart + i;
       PetscCall(MatSetValues_MPIAIJ(C, 1, &row, ncols, cols, vals, ADD_VALUES));
-      cols += ncols;
+      cols = PetscSafePointerPlusOffset(cols, ncols);
       vals += ncols;
     }
   } else {
